@@ -1,18 +1,46 @@
-## some convenience functions generally applicable to survreg
-## hopefully in survival soon
+## Dedicated methods for "tobit" objects that really should be inherited
+## from "survival". However, some versions of "survival" did not provide
+## these at all or had bugs. With survival >= 3.1-6 the methods in
+## "survival" are ok. So for now we still keep the "tobit" methods but
+## might remove them in future versions.
+
+fitted.tobit <- function(object, ...)
+  predict(object, type = "response", se.fit = FALSE)
+
+nobs.tobit <- function(object, ...)
+  length(object$linear.predictors)
+
+weights.tobit <- function(object, ...)
+  model.weights(model.frame(object))
+
+vcov.tobit <- function(object, ...) {
+  vc <- NextMethod()
+  if(is.null(colnames(vc))) {
+    nam <- names(object$coefficients)    
+    nam <- if(length(nam) == ncol(vc)) {
+      nam
+    } else if(length(nam) == ncol(vc) - 1L) {
+      c(nam, "Log(scale)")
+    } else {
+      c(nam, names(object$scale))
+    }
+    colnames(vc) <- rownames(vc) <- nam
+  }
+  return(vc)  
+}
+
+bread.tobit <- function(x, ...) {
+  length(x$linear.predictors) * vcov(x)
+}
+
+
+## "survival" chose not to include this deviance() method
+## so this needs to be provided even if "survival" >= 3.1-6
+## is required.
 
 deviance.survreg <- function(object, ...)
   sum(residuals(object, type = "deviance")^2)
 
-fitted.survreg <- function(object, ...)
-  predict(object, type = "response", se.fit = FALSE)
-
-nobs.survreg <- function(object, ...)
-  length(object$linear.predictors)
-
-weights.survreg <- function(object, ...)
-  model.weights(model.frame(object))
-
 
 ## convenience tobit() interface to survreg()
 
@@ -284,7 +312,7 @@ linearHypothesis.tobit <- function(model, hypothesis.matrix,
   } else {
     if(is.function(vcov.)) vcov. <- vcov.(model)
   }
-  vcov. <- vcov.[-nrow(vcov.), -ncol(vcov.)]
+  if("Log(scale)" %in% rownames(vcov.)) vcov. <- vcov.[-nrow(vcov.), -ncol(vcov.)]
   model$formula <- model$call$formula
   car::linearHypothesis.default(model,
     hypothesis.matrix = hypothesis.matrix, rhs = rhs, vcov. = vcov., ...)
